#!/usr/bin/python
#-----------------------------------------------------------------------
# File    : counts.py
# Contents: count patterns by size and support in independent data
# Author  : Christian Borgelt
# History : 2013.02.13 file created from detect/counts.py
#           2013.05.09 filling holes in pattern spectrum improved
#           2013.12.06 filling holes replaced by detection border
#           2014.05.27 modules placed in sibling directories
#           2015.09.08 completed adaptation to cover similarity
#-----------------------------------------------------------------------
from sys     import argv, path
from os      import makedirs
from os.path import join
from time    import strftime
from random  import seed
UTILDIR  = join('..', 'util')
if UTILDIR  not in path: path.insert(0, UTILDIR)
from id2par  import run
NEURODIR = join('..', 'neuro')
if NEURODIR not in path: path.insert(0, NEURODIR)
from psts    import genpsts
from ccnovl  import cocojim

#-----------------------------------------------------------------------

def counts (ncnt=100, period=3, rate=20,
            width=0.003, target='c', simcov='j',
            runs=1000, verbose=100, srng=1):
    if srng == 0: srng = 1      # init. seed and get data file name
    name = 'n%d_p%g_r%g_w%g_%c_%c' \
         % (ncnt,period,rate,width*10000,simcov,target)
    border = dict()             # create a support border
    hist   = dict()             # and a pattern count histogram
    for i in range(runs):       # repeat the experiments multiple times
        if verbose > 0 and i % verbose == 0:
            print(strftime('%Y.%m.%d %H:%M:%S counts ')+name+' '+str(i))
        seed(srng)              # seed random number generator
        k = int(ncnt/4); n = ncnt -3*k
        psts = genpsts(k, period, int(rate*0.4)) \
             + genpsts(k, period, int(rate*0.8)) \
             + genpsts(k, period, int(rate*1.2)) \
             + genpsts(n, period, int(rate*1.6))
        pats = cocojim(list(zip(range(1,ncnt+1), psts)),
                       target=target, width=width, supp=1.0,
                       simcov=simcov, sim=0.0, zmin=2,
                       mode='e', report='a')
        for p,c in pats:        # generate and mine spike trains
            z = len(p)          # update the support border
            if z not in border or c > border[z]: border[z] = c
            s = (z,int(c*1000)) # compute histogram bin
            if s in hist: hist[s] += 1
            else:         hist[s]  = 1
        srng += 2               # update the pattern histogram
    if verbose > 0:             # print termination log message
        print(strftime('%Y.%m.%d %H:%M:%S counts ')
                      +name+' '+str(runs)+' done.')
    try: makedirs(join('data','pc'))
    except OSError: pass        # ensure that directory exists
    with open(join('data','pc','bd_'+name+'.txt'), 'w') as out:
        for x in sorted(border.items()):
            out.write('%d %g\n' % x)
    n = 1.0/runs if runs > 0 else 1.0
    with open(join('data','pc','pc_'+name+'.txt'), 'w') as out:
        for x in sorted([(s[0],0.001*s[1],hist[s]*n) for s in hist]):
            out.write('%d %.3f %g\n' % x)
    return border               # write and return the support border

#-----------------------------------------------------------------------

if __name__ == '__main__':
    grids = [ [ ( 'ncnt',   [   100 ] ),
                ( 'period', [     3 ] ),
                ( 'rate',   [    20 ] ),
                ( 'target', [   'c' ] ),
                ( 'width',  [ 0.003, 0.0045 ] ),
                ( 'simcov', [ 'x', 'j', 'k', 'd', 's' ] ),
                ( 'runs',   [ 10000 ] ) ] ]

    run(counts, grids, int(argv[1])-1 if len(argv) > 1 else -1)
